#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Movable.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 1 + 34, "SCORE"); 
    PrintS(Vram + VramRowSize * 4 + 34, "HIGH");
    PrintS(Vram + VramRowSize * 5 + 34, "SCORE"); 
    PrintS(Vram + VramRowSize * 8 + 34, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 9 + 38, CurrentStage + 1);
    PrintS(Vram + VramRowSize * 11 + 34, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 22 + 34;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain);
                pVram += 2 * VramStep;
                *pVram = Char_Space; ++pVram;
                *pVram = Char_Space; ++pVram;
                *pVram = Char_Space; ++pVram;
                pVram += VramRowSize - 2 * VramStep;
                pVram = PrintC(pVram, i + '0');
                *pVram = Char_Space;
            }
            else {
                do {
                    Put2C(pVram, Char_Remain);
                    pVram += 2 * VramStep;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 2 + 34, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 6 + 34, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize * 12 + 37, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 10 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 12, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 13, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	C
        0x20, 0x2e, 0x25, 0x2b, 
        0x2c, 0x23, 0x20, 0x20, 
        0x24, 0x2b, 0x20, 0x2a, 
        0x20, 0x24, 0x25, 0x21, 
        //	R
        0x2c, 0x27, 0x25, 0x2b, 
        0x2c, 0x23, 0x28, 0x2f, 
        0x2c, 0x27, 0x2f, 0x22, 
        0x24, 0x21, 0x24, 0x25, 
        //	A
        0x20, 0x2e, 0x2d, 0x22, 
        0x2c, 0x23, 0x20, 0x2f, 
        0x2c, 0x27, 0x25, 0x2f, 
        0x24, 0x21, 0x20, 0x25, 
        //	C
        0x20, 0x2e, 0x25, 0x2b, 
        0x2c, 0x23, 0x20, 0x20, 
        0x24, 0x2b, 0x20, 0x2a, 
        0x20, 0x24, 0x25, 0x21, 
        //	K
        0x2c, 0x23, 0x28, 0x27, 
        0x2c, 0x2b, 0x27, 0x20, 
        0x2c, 0x23, 0x2d, 0x22, 
        0x24, 0x21, 0x20, 0x25, 
        //	Y
        0x2c, 0x23, 0x2c, 0x23, 
        0x24, 0x2b, 0x2e, 0x21, 
        0x20, 0x2c, 0x23, 0x20, 
        0x20, 0x24, 0x21, 0x20, 
    };
    constexpr byte LogoLength = 6;
    constexpr byte LogoWidth = LogoLength * 4;
    constexpr byte LogoX = (32 - LogoWidth) / 2;

    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 8 + LogoX;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
        
        pVram = Vram + VramRowSize * 8 + LogoX - 2;
        repeat (4) {
            pVram[0] = 9;
            pVram[1] = Color_Cyan;
            pVram[2 + LogoWidth] = 8;
            pVram[3 + LogoWidth] = Color_White;
            pVram += VramRowSize;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2023");
}
